@@ -1,3 +1,6 @@
+import json
+from typing import Dict
+
 import pytest
 
 import ddtrace
@@ -6,6 +9,7 @@ from ...constants import SPAN_KIND
 from ...ext import SpanTypes
 from ...ext import ci
 from ...ext import test
+from ...internal.logger import get_logger
 from ...pin import Pin
 from ..trace_utils import int_service
 from .constants import FRAMEWORK
@@ -13,6 +17,10 @@ from .constants import HELP_MSG
 from .constants import KIND
 
 
+PATCH_ALL_HELP_MSG = "Call ddtrace.patch_all before running tests."
+log = get_logger(__name__)
+
+
 def is_enabled(config):
     """Check if the ddtrace plugin is enabled."""
     return config.getoption("ddtrace") or config.getini("ddtrace")
@@ -28,9 +36,6 @@ def _store_span(item, span):
     setattr(item, "_datadog_span", span)
 
 
-PATCH_ALL_HELP_MSG = "Call ddtrace.patch_all before running tests."
-
-
 def pytest_addoption(parser):
     """Add ddtrace options."""
     group = parser.getgroup("ddtrace")
@@ -102,10 +107,21 @@ def pytest_runtest_protocol(item, nextitem):
         span.set_tag(test.SUITE, item.module.__name__)
         span.set_tag(test.TYPE, SpanTypes.TEST.value)
 
+        # Parameterized test cases will have a `callspec` attribute attached to the pytest Item object.
+        # Pytest docs: https://docs.pytest.org/en/6.2.x/reference.html#pytest.Function
+        if getattr(item, "callspec", None):
+            parameters = {"arguments": {}, "metadata": {}}  # type: Dict[str, Dict[str, str]]
+            for param_name, param_val in item.callspec.params.items():
+                try:
+                    parameters["arguments"][param_name] = repr(param_val)
+                except Exception:
+                    parameters["arguments"][param_name] = "Could not encode"
+                    log.warning("Failed to encode %r", param_name, exc_info=True)
+            span.set_tag(test.PARAMETERS, json.dumps(parameters))
+
         markers = [marker.kwargs for marker in item.iter_markers(name="dd_tags")]
         for tags in markers:
             span.set_tags(tags)
-
         _store_span(item, span)
 
         yield
@@ -132,8 +148,27 @@ def pytest_runtest_makereport(item, call):
 
     try:
         result = outcome.get_result()
+
+        if hasattr(result, "wasxfail") or "xfail" in result.keywords:
+            if result.skipped:
+                # XFail tests that fail are recorded skipped by pytest
+                span.set_tag(test.RESULT, test.Status.XFAIL.value)
+                span.set_tag(test.XFAIL_REASON, result.wasxfail)
+            else:
+                span.set_tag(test.RESULT, test.Status.XPASS.value)
+                if result.passed:
+                    # XPass (strict=False) are recorded passed by pytest
+                    span.set_tag(test.XFAIL_REASON, result.wasxfail)
+                else:
+                    # XPass (strict=True) are recorded failed by pytest, longrepr contains reason
+                    span.set_tag(test.XFAIL_REASON, result.longrepr)
+
         if result.skipped:
-            span.set_tag(test.STATUS, test.Status.SKIP.value)
+            if hasattr(result, "wasxfail"):
+                # XFail tests that fail are recorded skipped by pytest, should be passed instead
+                span.set_tag(test.STATUS, test.Status.PASS.value)
+            else:
+                span.set_tag(test.STATUS, test.Status.SKIP.value)
             reason = _extract_reason(call)
             if reason is not None:
                 span.set_tag(test.SKIP_REASON, reason)
